@@ -1,9 +1,14 @@
 error.prop <-
-function (data.obj, perm = FALSE, verbose = FALSE) {
+function (data.obj, pairscan.obj, perm = FALSE, verbose = FALSE, n.cores = NULL) {
+
+	p = NULL #for appeasing R CMD check
 
 	# require("qpcR")
 	# require("corpcor")
 
+	results.obj <- vector(mode = "list", length = 2)
+	names(results.obj) <- c("var.to.var.influences.perm")
+
 	if(verbose){
 		if(perm){
 			cat("\nCalculating error propagation of permuted coefficients.\n")
@@ -15,7 +20,7 @@ function (data.obj, perm = FALSE, verbose = FALSE) {
 	#====================================================================================
 	#begin internal functions
 	#====================================================================================
-		get.beta.mat <- function(scan.two.results, marker.pair.number){
+		get.beta.mat <- function(marker.pair.number){
 			#the beta matrix is composed of the coefficients from each pairwise
 			#marker model (except for the interaction coefficient)
 			#we use all markers in each row and set the non-covariate entries to 0
@@ -26,7 +31,7 @@ function (data.obj, perm = FALSE, verbose = FALSE) {
 			}
 
 
-		get.se.mat <- function(scan.two.results, marker.pair.number){
+		get.se.mat <- function(marker.pair.number){
 			#the beta matrix is composed of the coefficients from each pairwise
 			#marker model (except for the interaction coefficient)
 			#we use all markers in each row and set the non-covariate entries to 0
@@ -38,7 +43,7 @@ function (data.obj, perm = FALSE, verbose = FALSE) {
 
 
 		
-		get.cov.mat <- function(scan.two.results, marker.pair.number){
+		get.cov.mat <- function(marker.pair.number){
 			#the variance-covariance matrix is block diagonal, and 
 			#contains three times the number of rows and columns
 			#as scanned traits. The blocks contain the variance-
@@ -54,7 +59,7 @@ function (data.obj, perm = FALSE, verbose = FALSE) {
 			}
 	
 		#check to see if phenotypes or eigentraits were scanned
-		pheno.names <- names(data.obj$pairscan.results)	
+		pheno.names <- names(pairscan.obj$pairscan.results)	
 		pheno.check <- match(pheno.names, colnames(data.obj$pheno))
 		if(length(which(!is.na(pheno.check))) == 0){ #if we scanned eigentraits
 			num.pheno <- dim(data.obj$ET)[2] #the number of phenotypes
@@ -62,24 +67,40 @@ function (data.obj, perm = FALSE, verbose = FALSE) {
 			}else{
 			num.pheno <- dim(data.obj$pheno)[2] #the number of phenotypes
 			names.pheno <- colnames(data.obj$pheno)
-			}					
-
+			}				
+			
+		get.pair.coeffs <- function(marker.pair.number){
+			beta.main <- t(get.beta.mat(marker.pair.number)) ### Extract Main effect and interactions
+			non.zero <- which(beta.main[1:2,] != 0)
+			na.locale <- which(is.na(beta.main))
+			if(length(non.zero) > 0 && length(na.locale) == 0){
+				beta.se <- t(get.se.mat(marker.pair.number)) ### Extract Main effect and interactions
+				beta.cov <- get.cov.mat(marker.pair.number) ### Extract Covars
+				inf.coeffs <- calc.delta.errors(markers = marker.mat[marker.pair.number,], beta.m = beta.main, se = beta.se, beta.cov)
+				}else{
+				inf.coeffs <- NULL	
+				}
+			return(inf.coeffs)	
+			}	
+	#====================================================================================
+	#end internal functions
+	#====================================================================================
+		
 	### For all marker pairs calculate activity and IC
-	n.gene <- dim(data.obj$geno.for.pairscan)[2] #the number of genes used in the pairscan
 	
 	if(perm){
-		if(is.null(data.obj$pairscan.perm)){
+		if(is.null(pairscan.obj$pairscan.perm)){
 			stop("pairscan() with permutations must be run before error.prop()")
 			}
-		n.perm <- dim(data.obj$pairscan.perm[[1]][[1]])[1]/dim(data.obj$pairscan.results[[1]][[1]])[1]
-		marker.mat <-  data.obj$pairscan.perm[[1]][[1]][,1:2]#get all the pairs that were tested in the pair scan
-    	scan.two.results <- data.obj$pairscan.perm  #the coefficient matrices from the 2D scan			
+		n.perm <- dim(pairscan.obj$pairscan.perm[[1]][[1]])[1]/dim(data.obj$pairscan.results[[1]][[1]])[1]
+		marker.mat <-  pairscan.obj$pairscan.perm[[1]][[1]][,1:2]#get all the pairs that were tested in the pair scan
+    	scan.two.results <- pairscan.obj$pairscan.perm  #the coefficient matrices from the 2D scan			
 		}else{
-		if(is.null(data.obj$pairscan.results)){
+		if(is.null(pairscan.obj$pairscan.results)){
 			stop("pairscan() must be run before error.prop()")
 			}
-		marker.mat <- data.obj$pairscan.results[[1]][[1]][,1:2] #a matrix listing the names of used marker combinations
-	    scan.two.results <- data.obj$pairscan.results  #the coefficient matrices from the 2D scan
+		marker.mat <- pairscan.obj$pairscan.results[[1]][[1]][,1:2] #a matrix listing the names of used marker combinations
+	    scan.two.results <- pairscan.obj$pairscan.results  #the coefficient matrices from the 2D scan
 		}
 
     
@@ -87,24 +108,11 @@ function (data.obj, perm = FALSE, verbose = FALSE) {
 	n.pairs <- length(marker.mat[,1]) #number of pairs of genes
 
 
-	influence.coeffs <- matrix(NA, nrow = n.pairs, ncol = 6)
-	for(p in 1:length(marker.mat[,1])){
-
-		if(verbose){
-			report.progress(p, n.pairs, percent.text = 10, percent.dot = 2)
-			}
-
-
-		beta.main <- t(get.beta.mat(scan.two.results, p)) ### Extract Main effect and interactions
-		beta.se <- t(get.se.mat(scan.two.results, p)) ### Extract Main effect and interactions
-		beta.cov <- get.cov.mat(scan.two.results, p) ### Extract Covars
-		non.zero <- which(beta.main[1:2,] != 0)
-		if(length(non.zero) > 0){
-			influence.coeffs[p,] <- calc.delta.errors(marker.mat[p,], beta.main, beta.se, beta.cov)
-			}
-		 		 
-		 }
-	
+	registerDoParallel(cores = n.cores)
+	influence.coeffs <- foreach(p = 1:n.pairs, .combine = "rbind") %dopar% {
+		get.pair.coeffs(p)
+		}				
+		
 	colnames(influence.coeffs) <- c("marker1","marker2","m12","m12.std.dev","m21","m21.std.dev")
 
 	if(perm){
